-- AI Research Grid Addon for Elder Scrolls Online
-- Author: Stormknight/LCAmethyst
-- "All" and "Relative"  functionality by His Dad
-- Dwemer Motif by rkuhnjr
-- Dwemer Motif By garkin
-- French Translation by Motsah
-- Fixes by mpavlinsky
-- Close on Esc code by Wandamey
-- Glass, Xivkyn, Ancient Orc, and Mercenary Akaviri added in by Scinutz
-- Code clean & lot of stuff reworked by Ayantir

local ADDON_NAME = "AIResearchGrid"
local ADDON_VERSION = "5.1"
local ADDON_WEBSITE = "http://www.esoui.com/downloads/info265-AIResearchGrid.html"
local db

local playerName = GetUnitName("player")
local curCharacter = playerName
local currentCraft = CRAFTING_TYPE_WOODWORKING
local mergedCharacters = "#ALL"
local showRelative = false
local relative = {}
local maxColumns
local AIRG_UI = {}

local TRAIT_KNOWN = -1
local TRAIT_UNKNOWN = 0

-- Defaults options for saved variables
local defaults = {
	showMotifs = true,
	char = {
		[mergedCharacters] = {},
		[playerName] = {},
	},
}

-- Index of the line in our grid for a specific trait
local gridTraits = {
	[ITEM_TRAIT_TYPE_WEAPON_POWERED] = 1,
	[ITEM_TRAIT_TYPE_WEAPON_CHARGED] = 2,
	[ITEM_TRAIT_TYPE_WEAPON_PRECISE] = 3,
	[ITEM_TRAIT_TYPE_WEAPON_INFUSED] = 4,
	[ITEM_TRAIT_TYPE_WEAPON_DEFENDING] = 5,
	[ITEM_TRAIT_TYPE_WEAPON_TRAINING] = 6,
	[ITEM_TRAIT_TYPE_WEAPON_SHARPENED] = 7,
	[ITEM_TRAIT_TYPE_WEAPON_DECISIVE] = 8,
	[ITEM_TRAIT_TYPE_WEAPON_NIRNHONED] = 9,
	[ITEM_TRAIT_TYPE_ARMOR_STURDY] = 10,
	[ITEM_TRAIT_TYPE_ARMOR_IMPENETRABLE] = 11,
	[ITEM_TRAIT_TYPE_ARMOR_REINFORCED] = 12,
	[ITEM_TRAIT_TYPE_ARMOR_WELL_FITTED] = 13,
	[ITEM_TRAIT_TYPE_ARMOR_TRAINING] = 14,
	[ITEM_TRAIT_TYPE_ARMOR_INFUSED] = 15,
	[ITEM_TRAIT_TYPE_ARMOR_PROSPEROUS] = 16,
	[ITEM_TRAIT_TYPE_ARMOR_DIVINES] = 17,
	[ITEM_TRAIT_TYPE_ARMOR_NIRNHONED] = 18,
}

-- Display Order in tooltip
local styleChaptersLookup = {
	[1] = ITEM_STYLE_CHAPTER_AXES,
	[2] = ITEM_STYLE_CHAPTER_BELTS,
	[3] = ITEM_STYLE_CHAPTER_BOOTS,
	[4] = ITEM_STYLE_CHAPTER_BOWS,
	[5] = ITEM_STYLE_CHAPTER_CHESTS,
	[6] = ITEM_STYLE_CHAPTER_DAGGERS,
	[7] = ITEM_STYLE_CHAPTER_GLOVES,
	[8] = ITEM_STYLE_CHAPTER_HELMETS,
	[9] = ITEM_STYLE_CHAPTER_LEGS,
	[10] = ITEM_STYLE_CHAPTER_MACES,
	[11] = ITEM_STYLE_CHAPTER_SHIELDS,
	[12] = ITEM_STYLE_CHAPTER_SHOULDERS,
	[13] = ITEM_STYLE_CHAPTER_STAVES,
	[14] = ITEM_STYLE_CHAPTER_SWORDS,
}

-- These tables contain records of Styles in Display Order
-- Style Number, StyleIndex, Icon
local styleWithoutChapter = {
	{ITEMSTYLE_RACIAL_HIGH_ELF, 8},
	{ITEMSTYLE_RACIAL_DARK_ELF, 5},
	{ITEMSTYLE_RACIAL_WOOD_ELF, 9},
	{ITEMSTYLE_RACIAL_NORD, 6},
	{ITEMSTYLE_RACIAL_BRETON, 2},
	{ITEMSTYLE_RACIAL_REDGUARD, 3},
	{ITEMSTYLE_RACIAL_KHAJIIT, 10},
	{ITEMSTYLE_RACIAL_ORC, 4},
	{ITEMSTYLE_RACIAL_ARGONIAN, 7},
	{ITEMSTYLE_RACIAL_IMPERIAL, 35},
	{ITEMSTYLE_AREA_ANCIENT_ELF, 16},
	{ITEMSTYLE_AREA_REACH, 18},
	{ITEMSTYLE_ENEMY_DAEDRIC, 21},
	{ITEMSTYLE_ENEMY_PRIMITIVE, 20},
	{ITEMSTYLE_AREA_SOUL_SHRIVEN, 31},
}

-- Style Number, StyleIndex, Achievement ID
local styleWithChapter = {
	{ITEMSTYLE_AREA_DWEMER, 15, 1144},
	{ITEMSTYLE_GLASS, 29, 1319},
	{ITEMSTYLE_AREA_XIVKYN, 30, 1181},
	{ITEMSTYLE_AREA_ANCIENT_ORC, 23, 1341},
	{ITEMSTYLE_AREA_AKAVIRI, 34, 1318},
	{ITEMSTYLE_UNDAUNTED, 27, 1348},
	{ITEMSTYLE_DEITY_MALACATH, 14, 1412},
	{ITEMSTYLE_DEITY_TRINIMAC, 22, 1411},
	{ITEMSTYLE_ORG_OUTLAW, 48, 1417},
	{ITEMSTYLE_ALLIANCE_EBONHEART, 25, 1414},
	{ITEMSTYLE_ALLIANCE_ALDMERI, 26, 1415},
	{ITEMSTYLE_ALLIANCE_DAGGERFALL, 24, 1416},
	{ITEMSTYLE_ORG_ABAHS_WATCH, 42, 1422},
	{ITEMSTYLE_ORG_THIEVES_GUILD, 12, 1423},
	{ITEMSTYLE_ORG_ASSASSINS, 47, 1424},
	{ITEMSTYLE_ENEMY_DROMOTHRA, 46, 1659},
	{ITEMSTYLE_DEITY_AKATOSH, 17, 1660},
	{ITEMSTYLE_ORG_DARK_BROTHERHOOD, 13, 1661},
	{ITEMSTYLE_ENEMY_MINOTAUR, 40, 1662},
	{ITEMSTYLE_RAIDS_CRAGLORN, 28, 1714},
	{ITEMSTYLE_ENEMY_DRAUGR, 32, 1715},
	{ITEMSTYLE_AREA_YOKUDAN, 36, 1713},
	{ITEMSTYLE_UNUSED21, 59, 0}, -- Grim Harlequin
	{ITEMSTYLE_UNUSED22, 60, 1545}, -- Hollowjack
	--{ITEMSTYLE_ENEMY_SKINCHANGER, 43, 1676},
}

-- Set the icon highlights for the currently selected character
local function UpdateMotifsUI()

	-- Single Styles
	for styleIndex, known in ipairs(db.char[curCharacter].StyleSingle) do
	
		if not known then
			AIRG_UI.StyleSingleButtons[styleIndex]:SetColor(1, 0, 0, 1)
		else
			AIRG_UI.StyleSingleButtons[styleIndex]:SetColor(1, 1, 1, 1)
		end
		
	end
	
	-- =================
	-- Chapter Styles
	for styleIndex, styleData in ipairs(db.char[curCharacter].StyleChapter) do
		
		local tooltipText = ""
		local knownCount = 0
		local itemStyle = styleWithChapter[styleIndex][1]
		local stoneItemLink = GetSmithingStyleItemLink(styleWithChapter[styleIndex][2])
		local itemName = zo_strformat(SI_TOOLTIP_ITEM_NAME, GetItemLinkName(stoneItemLink))
		
		-- Run through the complete list in styleChaptersLookup, and look up to see which we have
		-- check how many chapters are known and build tooltip
		for chapterIndex, indexValue in ipairs(styleChaptersLookup) do
			if styleData[chapterIndex] then
				tooltipText = zo_strjoin(nil, tooltipText, "\n|cFFFFFF", GetString("SI_ITEMSTYLECHAPTER", indexValue), "|r")
				knownCount = knownCount + 1
			else
				tooltipText = zo_strjoin(nil, tooltipText, "\n|c806060", GetString("SI_ITEMSTYLECHAPTER", indexValue), "|r")
			end
		end

		if knownCount == 14 then
			AIRG_UI.StyleChapterButtons[styleIndex]:SetColor(1, 1, 1, 1)
		elseif knownCount > 0 then
			AIRG_UI.StyleChapterButtons[styleIndex]:SetColor(1, 1, 0, 1)
		else
			AIRG_UI.StyleChapterButtons[styleIndex]:SetColor(1, 0, 0, 1)
		end
		
		AIRG_UI.StyleChapterButtons[styleIndex].tooltipText = zo_strjoin(nil, zo_strformat("<<t:1>> (<<2>>/14)\n<<t:3>>\n", GetString("SI_ITEMSTYLE", itemStyle), knownCount, ZO_SELECTED_TEXT:Colorize(itemName)), tooltipText)
	
	end
	
	-- If a char hasn't been relogged since new style update
	if #db.char[curCharacter].StyleChapter < #styleWithChapter then
		for regulIndex = #db.char[curCharacter].StyleChapter + 1, #styleWithChapter do
			
			local itemStyle = styleWithChapter[regulIndex][1]
			local stoneItemLink = GetSmithingStyleItemLink(styleWithChapter[regulIndex][2])
			local itemName = zo_strformat(SI_TOOLTIP_ITEM_NAME, GetItemLinkName(stoneItemLink))
			
			AIRG_UI.StyleChapterButtons[regulIndex]:SetColor(1, 0, 0, 1)
			local tooltipText = ""
			for chapterIndex, indexValue in ipairs(styleChaptersLookup) do
				tooltipText = zo_strjoin(nil, tooltipText, "\n|c806060", GetString("SI_ITEMSTYLECHAPTER", indexValue), "|r")
			end
			
			AIRG_UI.StyleChapterButtons[regulIndex].tooltipText = zo_strjoin(nil, zo_strformat("<<t:1>> (<<2>>/14)\n<<t:3>>\n", GetString("SI_ITEMSTYLE", itemStyle), knownCount, ZO_SELECTED_TEXT:Colorize(itemName)), tooltipText)
			
		end
	end
	
end

-- Lookup the style data for the current character and send it to saved vars.
local function PopulateStyleData()

	-- Styles Wo Chapters
	db.char[curCharacter].StyleSingle = {}
	if not db.char[mergedCharacters].StyleSingle then db.char[mergedCharacters].StyleSingle = {} end
	
	for styleIndex, styleData in ipairs(styleWithoutChapter) do
		
		local isStyleKnown = IsSmithingStyleKnown(styleData[2], 1)
		db.char[curCharacter].StyleSingle[styleIndex] = isStyleKnown
		
		if db.char[mergedCharacters].StyleSingle[styleIndex] == nil or isStyleKnown then
			db.char[mergedCharacters].StyleSingle[styleIndex] = isStyleKnown
		end
		
	end
	
	-- Styles With Capters
	db.char[curCharacter].StyleChapter = {}
	if not db.char[mergedCharacters].StyleChapter then db.char[mergedCharacters].StyleChapter = {} end
	
	for styleIndex, styleData in ipairs (styleWithChapter) do
		
		db.char[curCharacter].StyleChapter[styleIndex] = {}
		if not db.char[mergedCharacters].StyleChapter[styleIndex] then db.char[mergedCharacters].StyleChapter[styleIndex] = {} end
		
		for chapterIndex, chapterValue in ipairs(styleChaptersLookup) do
			
			if styleData[3] ~= 0 then
			
				local _, numCompleted, numRequired = GetAchievementCriterion(styleData[3], chapterValue)
				db.char[curCharacter].StyleChapter[styleIndex][chapterValue] = numCompleted == numRequired
				
				if db.char[mergedCharacters].StyleChapter[styleIndex][chapterValue] == nil or numCompleted == numRequired then
					db.char[mergedCharacters].StyleChapter[styleIndex][chapterValue] = numCompleted == numRequired
				end
				
			elseif IsSmithingStyleKnown(styleData[2], 1) then
				db.char[curCharacter].StyleChapter[styleIndex][chapterValue] = true -- We assume a style without achievement is "full book"
			else
				db.char[curCharacter].StyleChapter[styleIndex][chapterValue] = false
			end
			
		end
	end
	
end

local function PopulateTraitDataForCraft(craftingType)
	
	db.char[playerName].traits[craftingType] = {}
	if not db.char[mergedCharacters].traits[craftingType] then db.char[mergedCharacters].traits[craftingType] = {} end
	
	for researchLineIndex = 1, GetNumSmithingResearchLines(craftingType) do
		
		db.char[playerName].traits[craftingType][researchLineIndex] = {}
		if not db.char[mergedCharacters].traits[craftingType][researchLineIndex] then db.char[mergedCharacters].traits[craftingType][researchLineIndex] = {} end
		
		local _, _, numTraits = GetSmithingResearchLineInfo(craftingType, researchLineIndex)
		
		for traitIndex = 1, numTraits do
		
			local traitType, _, known = GetSmithingResearchLineTraitInfo(craftingType, researchLineIndex, traitIndex)
			local _, timeRemainingSecs = GetSmithingResearchLineTraitTimes(craftingType, researchLineIndex, traitIndex)
			
			local rowNum = gridTraits[traitType]
			
			if known then
				db.char[playerName].traits[craftingType][researchLineIndex][rowNum] = TRAIT_KNOWN
				db.char[mergedCharacters].traits[craftingType][researchLineIndex][rowNum] = TRAIT_KNOWN
			else
				if timeRemainingSecs then
					local tTargetStamp = GetTimeStamp() + timeRemainingSecs
					db.char[playerName].traits[craftingType][researchLineIndex][rowNum] = tTargetStamp
					
					if db.char[mergedCharacters].traits[craftingType][researchLineIndex][rowNum] ~= TRAIT_KNOWN then
						db.char[mergedCharacters].traits[craftingType][researchLineIndex][rowNum] = tTargetStamp
					end
					
				else
					db.char[playerName].traits[craftingType][researchLineIndex][rowNum] = TRAIT_UNKNOWN
					if not db.char[mergedCharacters].traits[craftingType][researchLineIndex][rowNum] then db.char[mergedCharacters].traits[craftingType][researchLineIndex][rowNum] = TRAIT_UNKNOWN end
				end
			end
		end

	end
	
end

local function BuildRelative()

	local function BuildRelativeForCraft(charName, craftingType)
	
		relative[charName][craftingType] = {}
	
		for researchLineIndex = 1, GetNumSmithingResearchLines(craftingType) do
			
			relative[charName][craftingType][researchLineIndex] = {}
			local _, _, numTraits = GetSmithingResearchLineInfo(craftingType, researchLineIndex)
			
			for traitIndex = 1, numTraits do
			
				local traitType = GetSmithingResearchLineTraitInfo(craftingType, researchLineIndex, traitIndex)
				local rowNum = gridTraits[traitType]
				
				local isKnownForLocalPlayer = db.char[playerName].traits[craftingType][researchLineIndex][rowNum] == TRAIT_KNOWN
				local isKnownForRelative = db.char[charName].traits[craftingType][researchLineIndex][rowNum] == TRAIT_KNOWN
				
				if isKnownForLocalPlayer and not isKnownForRelative then
					relative[charName][craftingType][researchLineIndex][rowNum] = TRAIT_KNOWN
				else
					relative[charName][craftingType][researchLineIndex][rowNum] = TRAIT_UNKNOWN
				end
				
				local isInResearchForRelative = db.char[charName].traits[craftingType][researchLineIndex][rowNum] > TRAIT_UNKNOWN
				if isInResearchForRelative and db.char[charName].traits[craftingType][researchLineIndex][rowNum] <= GetTimeStamp() then
					db.char[charName].traits[craftingType][researchLineIndex][rowNum] = TRAIT_KNOWN
					db.char[mergedCharacters].traits[craftingType][researchLineIndex][rowNum] = TRAIT_KNOWN
				end
				
			end
			
		end
	end

	for charName, data in pairs(db.char) do
		if charName ~= mergedCharacters and charName ~= playerName then
			
			relative[charName] = {}
			
			BuildRelativeForCraft(charName, CRAFTING_TYPE_BLACKSMITHING)
			BuildRelativeForCraft(charName, CRAFTING_TYPE_CLOTHIER)
			BuildRelativeForCraft(charName, CRAFTING_TYPE_WOODWORKING)
			
		end
	end

end

local function BuildMatrix(eventCode, arg1, arg2, arg3)

	if eventCode == EVENT_SKILLS_FULL_UPDATE or eventCode == EVENT_PLAYER_ACTIVATED then
		
		-- Build is delayed to EVENT_PLAYER_ACTIVATED because of the ESO+ bonus which is only applied after the 1st load after OnAddonLoaded
		-- EVENT_SKILLS_FULL_UPDATE handle the % time bonus when buying a new skill in realtime
		
		db.char[playerName].traits = {}
		if not db.char[mergedCharacters].traits then db.char[mergedCharacters].traits = {} end
		
		PopulateTraitDataForCraft(CRAFTING_TYPE_BLACKSMITHING)
		PopulateTraitDataForCraft(CRAFTING_TYPE_CLOTHIER)
		PopulateTraitDataForCraft(CRAFTING_TYPE_WOODWORKING)
		
		BuildRelative()
		PopulateStyleData()
		UpdateMotifsUI()
		
		if eventCode == EVENT_PLAYER_ACTIVATED then
			EVENT_MANAGER:UnregisterForEvent(ADDON_NAME, EVENT_PLAYER_ACTIVATED) 
		end
		
	elseif eventCode == EVENT_SMITHING_TRAIT_RESEARCH_STARTED then
		local craftingType, researchLineIndex, traitIndex = arg1, arg2, arg3
		-- Set timer started for actual char and "ALL" for craftingType, researchLineIndex, traitIndex
		
		local traitType = GetSmithingResearchLineTraitInfo(craftingType, researchLineIndex, traitIndex)
		local rowNum = gridTraits[traitType]
		local _, timeRemainingSecs = GetSmithingResearchLineTraitTimes(craftingType, researchLineIndex, traitIndex)
		local tTargetStamp = GetTimeStamp() + timeRemainingSecs
		
		db.char[playerName].traits[craftingType][researchLineIndex][rowNum] = tTargetStamp
		
		if db.char[mergedCharacters].traits[craftingType][researchLineIndex][rowNum] ~= TRAIT_KNOWN then
			db.char[mergedCharacters].traits[craftingType][researchLineIndex][rowNum] = tTargetStamp
		end
		
	elseif eventCode == EVENT_SMITHING_TRAIT_RESEARCH_COMPLETED then
		local craftingType, researchLineIndex, traitIndex = arg1, arg2, arg3
		-- Set research completed for actual char and "ALL" for craftingType, researchLineIndex, traitIndex
		
		local traitType = GetSmithingResearchLineTraitInfo(craftingType, researchLineIndex, traitIndex)
		local rowNum = gridTraits[traitType]
		
		db.char[playerName].traits[craftingType][researchLineIndex][rowNum] = TRAIT_KNOWN
		db.char[mergedCharacters].traits[craftingType][researchLineIndex][rowNum] = TRAIT_KNOWN
		
	elseif eventCode == EVENT_STYLE_LEARNED then
	
		local styleIndex, chapterIndex = arg1, arg2
		-- Set style known for actual char and "ALL" for styleIndex, chapterIndex
		
		local updateUI
		-- Learned a complete book
		if chapterIndex == ITEM_STYLE_CHAPTER_ALL then
			
			-- Style without chapter
			for index, styleData in ipairs(styleWithoutChapter) do
				if styleData[2] == styleIndex then
					db.char[playerName].StyleSingle[index] = true
					db.char[mergedCharacters].StyleSingle[index] = true
					updateUI = true
					break
				end
			end
			
			-- Style with chapter in a complete book
			if not updateUI then
				for index, styleData in ipairs(styleWithChapter) do
					if styleData[2] == styleIndex then
						for indexChapter = 1, #db.char[playerName].StyleChapter[index] do
							db.char[playerName].StyleChapter[index][indexChapter] = true
							db.char[mergedCharacters].StyleChapter[index][indexChapter] = true
						end
						updateUI = true
						break
					end
				end
			end
			
		else
			
			local chapterLearned
			for index, chapterValue in ipairs(styleChaptersLookup) do
				if chapterValue == chapterIndex then
					chapterLearned = index
					break
				end
			end
			
			for index, styleData in ipairs(styleWithChapter) do
				if styleData[2] == styleIndex then
					db.char[playerName].StyleChapter[index][chapterLearned] = true
					db.char[mergedCharacters].StyleChapter[index][chapterLearned] = true
					updateUI = true
					break
				end
			end
			
		end
		
		if updateUI then
			UpdateMotifsUI()
		end
		
	end
	
end

-- User has clicked on one of the profession buttons
-- or we just want to refresh the display with current data
local function OnCraftSelected(thisCraft)

	if not thisCraft then
		if not currentCraft then
			return
		else
			thisCraft = currentCraft
		end
	end
	
	local filterTypeFromCraftingType = {
		[CRAFTING_TYPE_BLACKSMITHING] = ITEMFILTERTYPE_BLACKSMITHING,
		[CRAFTING_TYPE_CLOTHIER] = ITEMFILTERTYPE_CLOTHING,
		[CRAFTING_TYPE_WOODWORKING] = ITEMFILTERTYPE_WOODWORKING ,
	}
	
	AIRG_UI.WindowSubTitle:SetText(GetString("SI_ITEMFILTERTYPE", filterTypeFromCraftingType[thisCraft]))
	
	local researching
	local traitCount, alltraits, tooltiptext
	local maxLines = GetNumSmithingResearchLines(thisCraft) -- the number of columns for this profession
	
	currentCraft = thisCraft
	alltraits = 0
	
	for i = 1, maxColumns do
	
		researching = false
		
		for j = 1, 18 do
			AIRG_UI.gridButtons[i][j]:SetColor(1, 1, 1, 0.4)
			AIRG_UI.gridButtons[i][j]:SetTexture("/esoui/art/buttons/swatchframe_down.dds")
			AIRG_UI.gridButtons[i][j]:SetMouseEnabled(false)    -- effectively disable tooltip for this grid item.
		end
		
		if (i > maxLines) then
			AIRG_UI.columnButtons[i]:SetHidden(true)
			AIRG_UI.columnFooters[i]:SetText("")
		else
			
			local name, icon, _, _ = GetSmithingResearchLineInfo(thisCraft, i)  -- Get info on that specific item
			AIRG_UI.columnButtons[i]:SetNormalTexture(icon)
			AIRG_UI.columnButtons[i]:SetHidden(false)
			AIRG_UI.columnButtons[i].text = name
			traitCount = 0
			
			--Check for research expiry
			for rowNum, tKnown in pairs(db.char[curCharacter].traits[thisCraft][i]) do
				if (tKnown > 0 and tKnown < GetTimeStamp()) then
					db.char[curCharacter].traits[thisCraft][i][rowNum] = TRAIT_KNOWN  -- Change to known
				end
			end
			
			if showRelative then
				-- Great! relative mode! we need to address a matter of practical usage.
				--  We need to see that we actually can research that now, because we can't research more than one item type at a time.
				--  So we need to pre-check for item research and set the colours (of the texture) appropriately
				for rowNum,tKnown in pairs(db.char[curCharacter].traits[thisCraft][i]) do
					if (tKnown > 0) then  -- researching
						researching = true
					end
				end
				
				for rowNum, tKnown in pairs(relative[curCharacter][thisCraft][i]) do
					AIRG_UI.gridButtons[i][rowNum].Known = tKnown
					if (tKnown == TRAIT_KNOWN) then       -- Trait is known
						if researching then
							AIRG_UI.gridButtons[i][rowNum]:SetColor(0.7, 0.6, 0.6, .8)   -- Dull Reddish
						else
							AIRG_UI.gridButtons[i][rowNum]:SetColor(0.2, 1, 0.2, 1)   -- Green
							AIRG_UI.gridButtons[i][rowNum]:SetMouseEnabled(true)
						end
						tooltiptext = zo_strjoin(" ",AIRG_UI.rowLabels[rowNum]:GetText() , AIRG_UI.columnButtons[i].text)
						AIRG_UI.gridButtons[i][rowNum].tooltipText = tooltiptext
						AIRG_UI.gridButtons[i][rowNum]:SetTexture("/esoui/art/loot/loot_finesseitem.dds")
						traitCount = traitCount + 1
					end
				end
				
			else -- not relative
				for rowNum,tKnown in pairs(db.char[curCharacter].traits[thisCraft][i]) do
					AIRG_UI.gridButtons[i][rowNum].Known = tKnown
					
					if (tKnown == TRAIT_KNOWN) then       -- Trait is known
						AIRG_UI.gridButtons[i][rowNum]:SetColor(0.2, 1, 0.2, 1)   -- Green
						AIRG_UI.gridButtons[i][rowNum]:SetTexture("/esoui/art/loot/loot_finesseitem.dds")
						traitCount = traitCount + 1
						tooltiptext = zo_strjoin(" ",AIRG_UI.rowLabels[rowNum]:GetText() , AIRG_UI.columnButtons[i].text)
						AIRG_UI.gridButtons[i][rowNum].tooltipText = tooltiptext
						AIRG_UI.gridButtons[i][rowNum]:SetMouseEnabled(true)
					elseif (tKnown > 0) then   -- Trait is being researched
						AIRG_UI.gridButtons[i][rowNum]:SetTexture("/esoui/art/miscellaneous/gamepad/gp_icon_timer32.dds")
						AIRG_UI.gridButtons[i][rowNum]:SetColor(0.0, .5, .75, 1)   -- Blue
						AIRG_UI.gridButtons[i][rowNum]:SetMouseEnabled(true)
					else                        -- Trait is NOT known
						AIRG_UI.gridButtons[i][rowNum]:SetColor(1, 0.2, 0.2, 1)   -- Red
						AIRG_UI.gridButtons[i][rowNum]:SetTexture("esoui/art/buttons/decline_up.dds")  --  an   "X".
					end
				end
			end  --Relative

			AIRG_UI.columnFooters[i]:SetText(traitCount)
			alltraits = alltraits + traitCount
		end
	end
	
	AIRG_UI.columnFooterTitle:SetText(GetString(SI_CRAFTING_COMPONENT_TOOLTIP_TRAITS) .. "(" .. tostring(alltraits) .. ")")   -- "Traits"
	
end

local function ToggleMainWindow()

	if AIResearchGrid:IsHidden() then
		OnCraftSelected()
	end
	
	SCENE_MANAGER:ToggleTopLevel(AIResearchGrid)	-- Thanks Wandamey
	
end

-- Invoked when the user selected a character from the dropdown box
local function OnCharacterSelect(charName)

	if charName ~= GetString(AIRG_OPTIONS_ALL) then
		curCharacter = charName
	else
		curCharacter = mergedCharacters
	end
	
	-- The Relative Button doesn't make sense if selecting these
	if curCharacter == mergedCharacters or curCharacter == playerName then
		AIRG_UI.btnUpdateOther:SetState(BSTATE_DISABLED)
		showRelative = false
	else
		AIRG_UI.btnUpdateOther:SetState(BSTATE_NORMAL)
	end
	
	UpdateMotifsUI()
	OnCraftSelected()
	
end

-- Create AIRG UI
local function InitUI()
	
	-- Set the main window subtitle
	AIRG_UI.WindowSubTitle = GetControl(AIResearchGrid, "WindowSubTitle")
	
	-- Nice little line under the grid
	AIRG_UI.BottomDivider = GetControl(AIResearchGrid, "BottomDivider")
	AIRG_UI.BottomDivider:SetHidden(not db.showMotifs)
	
	-- CREATE the DROPDOWN BOX for CHARACTER SELECT
	-- Uses code based on example from Seerah
	AIRG_UI.charDropdown = WINDOW_MANAGER:CreateControlFromVirtual("AIResearchGridDropdownCharacter", AIResearchGrid, "ZO_StatsDropdownRow")
	AIRG_UI.charDropdown:SetWidth(205) -- Defaults to 607!!!
	AIRG_UI.charDropdown:SetAnchor(TOPRIGHT, AIResearchGrid, TOPRIGHT, -32, 0)
	AIRG_UI.charDropdown:GetNamedChild("Dropdown"):SetWidth(200)
	AIRG_UI.charDropdown.dropdown:SetSelectedItem(curCharacter)  -- Set the current character as selected
	
	local function OnItemSelect(_, choiceText, choice)  --this is the callback function for when an item gets selected in the dropdown
		OnCharacterSelect(choiceText)
	end
	
	for charName, _ in pairs(db.char) do
		if charName == mergedCharacters then
			charName = GetString(AIRG_OPTIONS_ALL)
		end
		local entry = AIRG_UI.charDropdown.dropdown:CreateItemEntry(charName, OnItemSelect)
		AIRG_UI.charDropdown.dropdown:AddItem(entry)
	end
	
	-- CREATE BUTTON FOR "Relative to" AT TOP_RIGHT
	AIRG_UI.btnUpdateOther = WINDOW_MANAGER:CreateControlFromVirtual("AIResearchGridButtonUpdateOther", AIResearchGrid, "ZO_DefaultButton")
	
	--Attempt to adjust button size to content, but units are different. May not be fixable. leave for meantime.
	local textlen = 0
	if string.len(GetString(AIRG_OPTIONS_RELATIVE)) > string.len(GetString(AIRG_OPTIONS_RELATIVE_TO))	then
		textlen = string.len(GetString(AIRG_OPTIONS_RELATIVE))
	else
		textlen = string.len(GetString(AIRG_OPTIONS_RELATIVE_TO))
	end

	AIRG_UI.btnUpdateOther:SetDimensions(140+textlen, 30)
	AIRG_UI.btnUpdateOther:SetAnchor(RIGHT, AIRG_UI.charDropdown, LEFT, 0, 0)
	AIRG_UI.btnUpdateOther:SetState(BSTATE_NORMAL)
	AIRG_UI.btnUpdateOther:SetMouseOverBlendMode(0)
	AIRG_UI.btnUpdateOther:SetMouseOverTexture("esoui/art/buttons/generic_highlight.dds")
	AIRG_UI.btnUpdateOther:SetClickSound("Click")
	AIRG_UI.btnUpdateOther:SetDisabledFontColor(ZO_DEFAULT_DISABLED_COLOR:UnpackRGBA())
	AIRG_UI.btnUpdateOther:SetFont("ZoFontGame")
	AIRG_UI.btnUpdateOther:SetText(GetString(AIRG_OPTIONS_RELATIVE_TO))
	AIRG_UI.btnUpdateOther:SetEnabled(false)  -- starts pointing to current character, which can't use this
	AIRG_UI.btnUpdateOther:SetHidden(false)
	-- enabled at end of function
	AIRG_UI.btnUpdateOther:SetHandler("OnClicked",
		function(self)
			showRelative = not showRelative
			if showRelative then
				self:SetText(GetString(AIRG_OPTIONS_RELATIVE))
			else
				self:SetText(GetString(AIRG_OPTIONS_RELATIVE_TO))
			end
			OnCraftSelected()
		end)

	AIRG_UI.btnUpdateOther:SetMouseEnabled(true)

	-- CREATE BUTTON FOR PROFESSION: BLACKSMITHING
	AIRG_UI.btnBlacksmithing = WINDOW_MANAGER:CreateControl("AIResearchGridButtonBlacksmithing", AIResearchGrid, CT_BUTTON)
	AIRG_UI.btnBlacksmithing:SetDimensions(48, 48)
	AIRG_UI.btnBlacksmithing:SetAnchor(TOPLEFT, AIResearchGrid, TOPLEFT, 8 , 44)
	AIRG_UI.btnBlacksmithing:SetState(BSTATE_NORMAL)
	AIRG_UI.btnBlacksmithing:SetMouseOverBlendMode(0)
	AIRG_UI.btnBlacksmithing:SetHidden(false)
	AIRG_UI.btnBlacksmithing:SetEnabled(true)
	AIRG_UI.btnBlacksmithing:SetClickSound(SOUNDS.WOODWORKER_EXTRACTED_BOOSTER)
	AIRG_UI.btnBlacksmithing:SetNormalTexture("/esoui/art/icons/ability_smith_007.dds")
	AIRG_UI.btnBlacksmithing:SetMouseOverTexture("esoui/art/buttons/generic_highlight.dds")
	AIRG_UI.btnBlacksmithing:SetHandler("OnClicked", function(self) OnCraftSelected(CRAFTING_TYPE_BLACKSMITHING) end)
	
	-- CREATE BUTTON FOR PROFESSION: WOODWORKING
	AIRG_UI.btnWoodworking = WINDOW_MANAGER:CreateControl("AIResearchGridButtonWoodworking", AIResearchGrid, CT_BUTTON)
	AIRG_UI.btnWoodworking:SetDimensions(48, 48)
	AIRG_UI.btnWoodworking:SetAnchor(TOPLEFT, AIRG_UI.btnBlacksmithing, TOPRIGHT, 12 , 0)
	AIRG_UI.btnWoodworking:SetState(BSTATE_NORMAL)
	AIRG_UI.btnWoodworking:SetMouseOverBlendMode(0)
	AIRG_UI.btnWoodworking:SetHidden(false)
	AIRG_UI.btnWoodworking:SetEnabled(true)
	AIRG_UI.btnWoodworking:SetClickSound(SOUNDS.BLACKSMITH_EXTRACTED_BOOSTER)
	AIRG_UI.btnWoodworking:SetNormalTexture("/esoui/art/icons/ability_tradecraft_009.dds")
	AIRG_UI.btnWoodworking:SetMouseOverTexture("esoui/art/buttons/generic_highlight.dds")
	AIRG_UI.btnWoodworking:SetHandler("OnClicked", function(self) OnCraftSelected(CRAFTING_TYPE_WOODWORKING) end)
	
	-- CREATE BUTTON FOR PROFESSION: CLOTHING
	AIRG_UI.btnClothing = WINDOW_MANAGER:CreateControl("AIResearchGridButtonClothing", AIResearchGrid, CT_BUTTON)
	AIRG_UI.btnClothing:SetDimensions(48, 48)
	AIRG_UI.btnClothing:SetAnchor(TOPLEFT, AIRG_UI.btnWoodworking, TOPRIGHT, 12 , 0)
	AIRG_UI.btnClothing:SetState(BSTATE_NORMAL)
	AIRG_UI.btnClothing:SetMouseOverBlendMode(0)
	AIRG_UI.btnClothing:SetHidden(false)
	AIRG_UI.btnClothing:SetEnabled(true)
	AIRG_UI.btnClothing:SetClickSound(SOUNDS.CLOTHIER_EXTRACTED_BOOSTER)
	AIRG_UI.btnClothing:SetNormalTexture("/esoui/art/icons/ability_tradecraft_008.dds")
	AIRG_UI.btnClothing:SetMouseOverTexture("esoui/art/buttons/generic_highlight.dds")
	AIRG_UI.btnClothing:SetHandler("OnClicked", function(self) OnCraftSelected(CRAFTING_TYPE_CLOTHIER) end)

	-- BUILD THE TRAIT LABELS ON THE LEFT-SIDE using built-in language strings
	AIRG_UI.rowLabels = {}
	for thisTrait, j in pairs(gridTraits) do
		AIRG_UI.rowLabels[j] = WINDOW_MANAGER:CreateControl("AIResearchGridRowLabel" .. tostring(j), AIResearchGrid, CT_LABEL)
		AIRG_UI.rowLabels[j]:SetAnchor(TOPLEFT, AIResearchGrid, TOPLEFT, 5, 28*j + 70)
		AIRG_UI.rowLabels[j]:SetText(GetString("SI_ITEMTRAITTYPE", thisTrait))       -- This is the text displayed on the screen as it's a label
		AIRG_UI.rowLabels[j]:SetDimensions(180, 24)
		AIRG_UI.rowLabels[j]:SetFont("ZoFontGame")
		AIRG_UI.rowLabels[j]:SetHorizontalAlignment(TEXT_ALIGN_RIGHT)
		AIRG_UI.rowLabels[j]:SetVerticalAlignment(TEXT_ALIGN_CENTER)
		AIRG_UI.rowLabels[j]:SetMouseEnabled(true)
		AIRG_UI.rowLabels[j]:SetHandler("OnMouseEnter", function (self)
			ZO_Tooltips_ShowTextTooltip(self, TOP, self.tooltipText)
		end)
		AIRG_UI.rowLabels[j]:SetHandler("OnMouseExit", function (self)
			ZO_Tooltips_HideTextTooltip()
		end)
	end
	
	-- Now add tooltip text to the row labels.
	-- The tooltip functionality is automatic as part of the label control.
	for j = 1, 9 do
		local tTypeW, tDescW, _ = GetSmithingResearchLineTraitInfo(CRAFTING_TYPE_BLACKSMITHING,1,j) -- weapons
		AIRG_UI.rowLabels[gridTraits[tTypeW]].tooltipText = tDescW
		local tTypeA, tDescA, _ = GetSmithingResearchLineTraitInfo(CRAFTING_TYPE_BLACKSMITHING,9,j) -- armour
		AIRG_UI.rowLabels[gridTraits[tTypeA]].tooltipText = tDescA
	end
	
	-- BUILD THE COLUMNS & GRID
	-- Determine the maximum number of item types across all three professions. This is currently always 14
	maxColumns = math.max(GetNumSmithingResearchLines(CRAFTING_TYPE_BLACKSMITHING),
	GetNumSmithingResearchLines(CRAFTING_TYPE_CLOTHIER),
	GetNumSmithingResearchLines(CRAFTING_TYPE_WOODWORKING))
	AIRG_UI.columnButtons = {}
	AIRG_UI.gridButtons = {}
	AIRG_UI.columnFooters = {}
	
	for i = 1, maxColumns do
		-- BUILD THE COLUMN BUTTONS
		AIRG_UI.columnButtons[i] = WINDOW_MANAGER:CreateControl("AIResearchGridHeaderButton" .. tostring(i), AIResearchGrid, CT_BUTTON)
		AIRG_UI.columnButtons[i]:SetDimensions(36, 36)
		AIRG_UI.columnButtons[i]:SetState(BSTATE_NORMAL)
		AIRG_UI.columnButtons[i]:SetAnchor(TOPLEFT, AIResearchGrid, TOPLEFT, 40*i + 150, 55)
		AIRG_UI.columnButtons[i]:SetHidden(true)
		AIRG_UI.columnButtons[i]:SetEnabled(true)
		AIRG_UI.columnButtons[i]:SetMouseOverTexture("esoui/art/buttons/generic_highlight.dds")
		AIRG_UI.columnButtons[i].text = i	-- placeholder
		AIRG_UI.columnButtons[i]:SetHandler("OnMouseEnter", function (self)
			ZO_Tooltips_ShowTextTooltip(self, TOP, self.text)
		end)
		AIRG_UI.columnButtons[i]:SetHandler("OnMouseExit", function (self)
			ZO_Tooltips_HideTextTooltip()
		end)
		-- BUILD THE GRID
		-- There are a total of 18 traits possible. 9 armour and 9 weapon (Infused, Training and Nirnhoned on both)
		AIRG_UI.gridButtons[i] = {}
		for j = 1, 18 do
			AIRG_UI.gridButtons[i][j] = WINDOW_MANAGER:CreateControl("AIResearchGridGridButton" .. tostring(i) .. "x" .. tostring(j), AIResearchGrid, CT_TEXTURE)
			AIRG_UI.gridButtons[i][j]:SetDimensions(24, 24)
			AIRG_UI.gridButtons[i][j]:SetAnchor(TOP, AIRG_UI.columnButtons[i], BOTTOM, 0, 28*j -20)
			AIRG_UI.gridButtons[i][j]:SetTexture("/esoui/art/buttons/swatchframe_down.dds")  -- little square box
			AIRG_UI.gridButtons[i][j]:SetColor(1, 1, 1, 0.4)
			AIRG_UI.gridButtons[i][j]:SetHidden(false)
			AIRG_UI.gridButtons[i][j]:SetMouseEnabled(false)
			-- Following tooltip code uses ideas contributed by Krysstof
			AIRG_UI.gridButtons[i][j]:SetHandler("OnMouseEnter", function (self)
				--   if (self.tooltipText > 0) then      -- Check the item is being researched and has a timestamp
				if (self.Known > 0) then      -- Check the item is being researched and has a timestamp
					local tRemaining = tonumber(self.Known) - GetTimeStamp()
					local tFormatted = FormatTimeSeconds(tRemaining, TIME_FORMAT_STYLE_DESCRIPTIVE_SHORT, TIME_FORMAT_PRECISION_SECONDS, TIME_FORMAT_DIRECTION_DESCENDING)
					ZO_Tooltips_ShowTextTooltip(self, RIGHT, tFormatted)
					-- loop with "registerforupdate" once per second
					EVENT_MANAGER:RegisterForUpdate(ADDON_NAME, 1000, function()
						local tRemaining = tonumber(self.Known) - GetTimeStamp()
						local tFormatted = FormatTimeSeconds(tRemaining, TIME_FORMAT_STYLE_DESCRIPTIVE_SHORT, TIME_FORMAT_PRECISION_SECONDS, TIME_FORMAT_DIRECTION_DESCENDING)
						ZO_Tooltips_ShowTextTooltip(self, RIGHT, tFormatted)
					end)
				else
					ZO_Tooltips_ShowTextTooltip(self, RIGHT, self.tooltipText)
				end
			end)
			AIRG_UI.gridButtons[i][j]:SetHandler("OnMouseExit", function (self)
				-- unregister the update event or it keeps on displaying
				EVENT_MANAGER:UnregisterForUpdate(ADDON_NAME)
				ZO_Tooltips_HideTextTooltip()
			end)
		end
		
		-- BUILD THE COLUMN FOOTERS
		AIRG_UI.columnFooters[i] = WINDOW_MANAGER:CreateControl("AIResearchGridColumnFooterLabel" .. tostring(i), AIResearchGrid, CT_LABEL)
		AIRG_UI.columnFooters[i]:SetAnchor(TOP, AIRG_UI.columnButtons[i], BOTTOM, 0, 516)
		AIRG_UI.columnFooters[i]:SetDimensions(36, 24)
		AIRG_UI.columnFooters[i]:SetFont("ZoFontGame")
		AIRG_UI.columnFooters[i]:SetHorizontalAlignment(TEXT_ALIGN_CENTER)
		AIRG_UI.columnFooters[i]:SetVerticalAlignment(TEXT_ALIGN_CENTER)
		AIRG_UI.columnFooters[i]:SetColor(1, 1, 0.4, 1);    -- faded yellow
		
	end
	
	-- BUILD LABEL FOR COLUMN FOOTER "TRAIT LINE"
	AIRG_UI.columnFooterTitle = WINDOW_MANAGER:CreateControl("AIResearchGridColumnFooterTitleLabel", AIResearchGrid, CT_LABEL)
	AIRG_UI.columnFooterTitle:SetAnchor(TOPLEFT, AIResearchGrid, TOPLEFT, 5, 606)
	AIRG_UI.columnFooterTitle:SetDimensions(180, 24)
	AIRG_UI.columnFooterTitle:SetFont("ZoFontGame")
	AIRG_UI.columnFooterTitle:SetHorizontalAlignment(TEXT_ALIGN_RIGHT)
	AIRG_UI.columnFooterTitle:SetVerticalAlignment(TEXT_ALIGN_CENTER)
	AIRG_UI.columnFooterTitle:SetColor(1, 1, 0.4, 1);       -- faded yellow
	AIRG_UI.columnFooterTitle:SetText(GetString(SI_CRAFTING_COMPONENT_TOOLTIP_TRAITS))   -- "Traits"
	
	AIResearchGrid:SetHeight(db.showMotifs and 800 or 636)
	
	-- BUILD THE MOTIF ICONS ACROSS THE BOTTOM
	-- It's set-up inside a container frame to make hiding or showing the whole lot simpler.
	AIRG_UI.motifSection = WINDOW_MANAGER:CreateControl("AIResearchMotifSection", AIResearchGrid, CT_CONTROL)
	AIRG_UI.motifSection:SetDimensions(750, 150)
	AIRG_UI.motifSection:SetAnchor(BOTTOMLEFT, AIResearchGrid, BOTTOMLEFT, 0, 0)
	AIRG_UI.motifSection:SetHidden(not db.showMotifs)

	AIRG_UI.StyleSingleButtons = {}
	for styleIndex, styleData in ipairs(styleWithoutChapter) do
		AIRG_UI.StyleSingleButtons[styleIndex] = WINDOW_MANAGER:CreateControl("AIRGStyleSingleButton" .. tostring(styleIndex), AIRG_UI.motifSection, CT_TEXTURE)
		AIRG_UI.StyleSingleButtons[styleIndex]:SetDimensions(40, 40)
		AIRG_UI.StyleSingleButtons[styleIndex]:SetAnchor(TOPLEFT, AIRG_UI.motifSection, TOPLEFT, 40*styleIndex+40, 0)
		
		local stoneItemLink = GetSmithingStyleItemLink(styleData[2])
		local itemName = zo_strformat(SI_TOOLTIP_ITEM_NAME, GetItemLinkName(stoneItemLink))
		
		AIRG_UI.StyleSingleButtons[styleIndex]:SetTexture(GetItemLinkInfo(stoneItemLink))
		AIRG_UI.StyleSingleButtons[styleIndex]:SetMouseEnabled(true)
		AIRG_UI.StyleSingleButtons[styleIndex].tooltipText = zo_strformat("<<t:1>>\n<<t:2>>", GetString("SI_ITEMSTYLE",styleData[1]), itemName)
		AIRG_UI.StyleSingleButtons[styleIndex]:SetHandler("OnMouseEnter", function (self)
			ZO_Tooltips_ShowTextTooltip(self, TOP, self.tooltipText)
		end)
		AIRG_UI.StyleSingleButtons[styleIndex]:SetHandler("OnMouseExit", function (self)
			ZO_Tooltips_HideTextTooltip()
		end)
	end

	AIRG_UI.StyleChapterButtons = {}
	
	local yStyleAnchor = 50
	local xStyleAnchor = 1
	local xStyleAnchorAlign = 60
	local secondLine1stIndex = 15
	
	for styleIndex, styleData in ipairs(styleWithChapter) do
	
		if styleIndex == secondLine1stIndex then
			xStyleAnchor = 1
			yStyleAnchor = 100
			xStyleAnchorAlign = 140
		end
	
		AIRG_UI.StyleChapterButtons[styleIndex] = WINDOW_MANAGER:CreateControl("AIRGStyleChapterButton" .. tostring(styleIndex), AIRG_UI.motifSection, CT_TEXTURE)
		AIRG_UI.StyleChapterButtons[styleIndex]:SetDimensions(40, 40)
		AIRG_UI.StyleChapterButtons[styleIndex]:SetAnchor(TOPLEFT, AIRG_UI.motifSection, TOPLEFT, 40 * xStyleAnchor + xStyleAnchorAlign, yStyleAnchor)
		
		local stoneItemLink = GetSmithingStyleItemLink(styleData[2])
		local itemName = zo_strformat(SI_TOOLTIP_ITEM_NAME, GetItemLinkName(stoneItemLink))
		
		AIRG_UI.StyleChapterButtons[styleIndex]:SetTexture(GetItemLinkInfo(stoneItemLink))
		AIRG_UI.StyleChapterButtons[styleIndex]:SetMouseEnabled(true)
		AIRG_UI.StyleChapterButtons[styleIndex].tooltipText = ""
		AIRG_UI.StyleChapterButtons[styleIndex]:SetHandler("OnMouseEnter", function (self)
			ZO_Tooltips_ShowTextTooltip(self, TOP, self.tooltipText)
		end)
		AIRG_UI.StyleChapterButtons[styleIndex]:SetHandler("OnMouseExit", function (self)
			ZO_Tooltips_HideTextTooltip()
		end)
		xStyleAnchor = xStyleAnchor + 1
	end
	
	-- BUILD LABEL FOR STYLES
	AIRG_UI.StyleLabel = WINDOW_MANAGER:CreateControl("AIResearchStylelabel", AIRG_UI.motifSection, CT_LABEL)
	AIRG_UI.StyleLabel:SetAnchor(TOPLEFT, AIRG_UI.motifSection, TOPLEFT, 5, 6)
	AIRG_UI.StyleLabel:SetText("")
	AIRG_UI.StyleLabel:SetDimensions(150, 24)
	AIRG_UI.StyleLabel:SetFont("ZoFontGame")
	AIRG_UI.StyleLabel:SetHorizontalAlignment(TEXT_ALIGN_RIGHT)

end

local function CreateConfigMenu()

	local panelData = {
		type = "panel",
		name = "AI Research Grid",
		displayName = "|c8080FFAI Research Grid|r",
		author = "Ayantir & others",
		version = ADDON_VERSION,
		website = ADDON_WEBSITE,
	}
	
	local optionsData = {
		{
			type = "header",
			name = GetString(AIRG_OPTIONS_GENERAL),
		},
		{
			type = "checkbox",
			name = GetString(AIRG_OPTIONS_MOTIFS),
			tooltip = GetString(AIRG_OPTIONS_MOTIFS_DESC),
			getFunc = function() return db.showMotifs end,
			setFunc = function(newValue)
				db.showMotifs = newValue
				AIRG_UI.motifSection:SetHidden(not newValue)
				AIRG_UI.BottomDivider:SetHidden(not newValue)
				AIResearchGrid:SetHeight(newValue and 800 or 636)
			end,
		},
		{
			type = "header",
			name = GetString(AIRG_OPTIONS_KEYBINDS),
		},
		{
			type = "description",
			text = GetString(AIRG_OPTIONS_KEYBIND_DESC),
		},
	}

	local LAM2 = LibStub("LibAddonMenu-2.0")
	LAM2:RegisterAddonPanel(ADDON_NAME .. "Options", panelData)
	LAM2:RegisterOptionControls(ADDON_NAME .. "Options", optionsData)

end

local function OnAddonLoaded(_, addOnName)

	-- Only initialize our own addon
	if ADDON_NAME == addOnName then
	
		SLASH_COMMANDS["/airg"] = ToggleMainWindow
		
		-- Load the saved variables
		db = ZO_SavedVars:NewAccountWide("AIRG_SavedVariables", 3, nil, defaults)
		
		-- Register Keybinding
		ZO_CreateStringId("SI_BINDING_NAME_TOGGLE_AIRG", GetString(AIRG_KEYBIND_TOGGLE))
		
		-- Character renamed
		if NAME_CHANGE:DidNameChange() then
			db.char[GetUnitName("player")] = db.char[NAME_CHANGE:GetOldCharacterName()]
			db.char[NAME_CHANGE:GetOldCharacterName()] = nil
		end
		
		-- Add id to protect
		for i = 1, GetNumCharacters() do
			local name, _, _, _, _, _, characterId = GetCharacterInfo(i)
			name = zo_strformat(SI_UNIT_NAME, name)
			if db.char[name] and not db.char[name].id then
				db.char[name].id = characterId
			end
		end
		
		-- No id = Char deleted
		for charName, charData in pairs(db.char) do
			if charName ~= mergedCharacters and charName ~= GetUnitName("player") and not charData.id then
				db.char[charName] = nil
			end
		end
		
		-- UI set-up. Create frames, position labels & buttons etc
		InitUI()
		
		-- Create the configuration/settings menu
		CreateConfigMenu()
		
		EVENT_MANAGER:RegisterForEvent(ADDON_NAME, EVENT_SMITHING_TRAIT_RESEARCH_STARTED, BuildMatrix)
		EVENT_MANAGER:RegisterForEvent(ADDON_NAME, EVENT_SMITHING_TRAIT_RESEARCH_COMPLETED, BuildMatrix)
		EVENT_MANAGER:RegisterForEvent(ADDON_NAME, EVENT_SKILLS_FULL_UPDATE, BuildMatrix)
		EVENT_MANAGER:RegisterForEvent(ADDON_NAME, EVENT_PLAYER_ACTIVATED, BuildMatrix)
		EVENT_MANAGER:RegisterForEvent(ADDON_NAME, EVENT_STYLE_LEARNED, BuildMatrix)
		
		SCENE_MANAGER:RegisterTopLevel(AIResearchGrid, false)		-- enables close on Esc
		EVENT_MANAGER:UnregisterForEvent(ADDON_NAME, EVENT_ADD_ON_LOADED)
	
	end
	
end

function AIRG_ToggleMainWindow()
	ToggleMainWindow()
end

EVENT_MANAGER:RegisterForEvent(ADDON_NAME, EVENT_ADD_ON_LOADED, OnAddonLoaded)